@@ -15,11 +15,15 @@
  * @category   Zend
  * @package    Zend_Amf
  * @subpackage Parse_Amf3
- * @copyright  Copyright (c) 2005-2009 Zend Technologies USA Inc. (http://www.zend.com)
+ * @copyright  Copyright (c) 2005-2010 Zend Technologies USA Inc. (http://www.zend.com)
  * @license    http://framework.zend.com/license/new-bsd     New BSD License
- * @version    $Id: Serializer.php 16971 2009-07-22 18:05:45Z mikaelkael $
+ * @version    $Id: Serializer.php 21969 2010-04-22 03:54:59Z matthew $
  */
 
+/** Zend_Amf_Constants */
+require_once 'Zend/Amf/Constants.php';
+
+
 /** Zend_Amf_Parse_Serializer */
 require_once 'Zend/Amf/Parse/Serializer.php';
 
@@ -31,11 +35,17 @@ require_once 'Zend/Amf/Parse/TypeLoader.php';
  *
  * @package    Zend_Amf
  * @subpackage Parse_Amf3
- * @copyright  Copyright (c) 2005-2009 Zend Technologies USA Inc. (http://www.zend.com)
+ * @copyright  Copyright (c) 2005-2010 Zend Technologies USA Inc. (http://www.zend.com)
  * @license    http://framework.zend.com/license/new-bsd     New BSD License
  */
 class Zend_Amf_Parse_Amf3_Serializer extends Zend_Amf_Parse_Serializer
 {
+    /**
+     * A constant empty string
+     * @var string
+     */
+    protected $_strEmpty = '';
+    
     /**
      * An array of reference objects per amf body
      * @var array
@@ -47,13 +57,13 @@ class Zend_Amf_Parse_Amf3_Serializer extends Zend_Amf_Parse_Serializer
      * @var array
      */
     protected $_referenceStrings = array();
-    
+
     /**
      * An array of reference class definitions, indexed by classname
      * @var array
      */
     protected $_referenceDefinitions = array();
-    
+
     /**
      * Serialize PHP types to AMF3 and write to stream
      *
@@ -61,12 +71,18 @@ class Zend_Amf_Parse_Amf3_Serializer extends Zend_Amf_Parse_Serializer
      * auto negotiates the type or use the user defined markerType to
      * serialize the data from php back to AMF3
      *
-     * @param  mixed $content
+     * @param  mixed $data
      * @param  int $markerType
+     * @param  mixed $dataByVal
      * @return void
      */
-    public function writeTypeMarker($data, $markerType=null)
+    public function writeTypeMarker(&$data, $markerType = null, $dataByVal = false)
     {
+        // Workaround for PHP5 with E_STRICT enabled complaining about "Only 
+        // variables should be passed by reference"
+        if ((null === $data) && ($dataByVal !== false)) {
+            $data = &$dataByVal;
+        }
         if (null !== $markerType) {
             // Write the Type Marker to denote the following action script data type
             $this->_stream->writeByte($markerType);
@@ -108,11 +124,11 @@ class Zend_Amf_Parse_Amf3_Serializer extends Zend_Amf_Parse_Serializer
             }
         } else {
             // Detect Type Marker
-            if(is_resource($data)) {
+            if (is_resource($data)) {
                 $data = Zend_Amf_Parse_TypeLoader::handleResource($data);
             }
-             switch (true) {
-             	case (null === $data):
+            switch (true) {
+                case (null === $data):
                     $markerType = Zend_Amf_Constants::AMF3_NULL;
                     break;
                 case (is_bool($data)):
@@ -150,10 +166,10 @@ class Zend_Amf_Parse_Amf3_Serializer extends Zend_Amf_Parse_Serializer
                         $markerType = Zend_Amf_Constants::AMF3_OBJECT;
                     }
                     break;
-                default: 
+                default:
                     require_once 'Zend/Amf/Exception.php';
                     throw new Zend_Amf_Exception('Unsupported data type: ' . gettype($data));
-             }
+            }
             $this->writeTypeMarker($data, $markerType);
         }
     }
@@ -190,7 +206,7 @@ class Zend_Amf_Parse_Amf3_Serializer extends Zend_Amf_Parse_Serializer
         $this->_stream->writeByte($int & 0xff);
         return $this;
     }
-    
+
     /**
      * Send string to output stream, without trying to reference it.
      * The string is prepended with strlen($string) << 1 | 0x01
@@ -198,11 +214,11 @@ class Zend_Amf_Parse_Amf3_Serializer extends Zend_Amf_Parse_Serializer
      * @param  string $string
      * @return Zend_Amf_Parse_Amf3_Serializer
      */
-    protected function writeBinaryString($string){
+    protected function writeBinaryString(&$string){
         $ref = strlen($string) << 1 | 0x01;
         $this->writeInteger($ref);
         $this->_stream->writeBytes($string);
-        
+
         return $this;
     }
 
@@ -212,14 +228,14 @@ class Zend_Amf_Parse_Amf3_Serializer extends Zend_Amf_Parse_Serializer
      * @param  string $string
      * @return Zend_Amf_Parse_Amf3_Serializer
      */
-    public function writeString($string)
+    public function writeString(&$string)
     {
         $len = strlen($string);
         if(!$len){
             $this->writeInteger(0x01);
             return $this;
         }
-        
+
         $ref = array_search($string, $this->_referenceStrings, true);
         if($ref === false){
             $this->_referenceStrings[] = $string;
@@ -228,22 +244,23 @@ class Zend_Amf_Parse_Amf3_Serializer extends Zend_Amf_Parse_Serializer
             $ref <<= 1;
             $this->writeInteger($ref);
         }
-        
+
         return $this;
     }
-    
+
     /**
      * Send ByteArray to output stream
      *
      * @param  string|Zend_Amf_Value_ByteArray  $data
      * @return Zend_Amf_Parse_Amf3_Serializer
      */
-    public function writeByteArray($data){
-        if($this->writeObjectReference($data)){
+    public function writeByteArray(&$data)
+    {
+        if ($this->writeObjectReference($data)) {
             return $this;
         }
-        
-        if(is_string($data)) {
+
+        if (is_string($data)) {
             //nothing to do
         } else if ($data instanceof Zend_Amf_Value_ByteArray) {
             $data = $data->getData();
@@ -251,13 +268,13 @@ class Zend_Amf_Parse_Amf3_Serializer extends Zend_Amf_Parse_Serializer
             require_once 'Zend/Amf/Exception.php';
             throw new Zend_Amf_Exception('Invalid ByteArray specified; must be a string or Zend_Amf_Value_ByteArray');
         }
-        
+
         $this->writeBinaryString($data);
-        
+
         return $this;
     }
-    
-	/**
+
+    /**
      * Send xml to output stream
      *
      * @param  DOMDocument|SimpleXMLElement  $xml
@@ -265,9 +282,9 @@ class Zend_Amf_Parse_Amf3_Serializer extends Zend_Amf_Parse_Serializer
      */
     public function writeXml($xml)
     {
-        if($this->writeObjectReference($xml)){
+        if ($this->writeObjectReference($xml)) {
             return $this;
-        }    
+        }
 
         if(is_string($xml)) {
             //nothing to do
@@ -279,9 +296,9 @@ class Zend_Amf_Parse_Amf3_Serializer extends Zend_Amf_Parse_Serializer
             require_once 'Zend/Amf/Exception.php';
             throw new Zend_Amf_Exception('Invalid xml specified; must be a DOMDocument or SimpleXMLElement');
         }
-        
+
         $this->writeBinaryString($xml);
-        
+
         return $this;
     }
 
@@ -293,10 +310,10 @@ class Zend_Amf_Parse_Amf3_Serializer extends Zend_Amf_Parse_Serializer
      */
     public function writeDate($date)
     {
-        if($this->writeObjectReference($date)){
+        if ($this->writeObjectReference($date)) {
             return $this;
         }
-        
+
         if ($date instanceof DateTime) {
             $dateString = $date->format('U') * 1000;
         } elseif ($date instanceof Zend_Date) {
@@ -318,16 +335,15 @@ class Zend_Amf_Parse_Amf3_Serializer extends Zend_Amf_Parse_Serializer
      * @param array $array
      * @return Zend_Amf_Parse_Amf3_Serializer
      */
-    public function writeArray(array $array)
+    public function writeArray(&$array)
     {
-        if($this->writeObjectReference($array)){
-            return $this;
-        }
-        
+        // arrays aren't reference here but still counted
+        $this->_referenceObjects[] = $array;
+
         // have to seperate mixed from numberic keys.
         $numeric = array();
         $string  = array();
-        foreach ($array as $key => $value) {
+        foreach ($array as $key => &$value) {
             if (is_int($key)) {
                 $numeric[] = $value;
             } else {
@@ -341,30 +357,39 @@ class Zend_Amf_Parse_Amf3_Serializer extends Zend_Amf_Parse_Serializer
         $this->writeInteger($id);
 
         //Write the mixed type array to the output stream
-        foreach($string as $key => $value) {
+        foreach($string as $key => &$value) {
             $this->writeString($key)
                  ->writeTypeMarker($value);
         }
-        $this->writeString('');
+        $this->writeString($this->_strEmpty);
 
         // Write the numeric array to ouput stream
-        foreach($numeric as $value) {
+        foreach($numeric as &$value) {
             $this->writeTypeMarker($value);
         }
         return $this;
     }
-    
+
     /**
      * Check if the given object is in the reference table, write the reference if it exists,
      * otherwise add the object to the reference table
-     * 
-     * @param mixed $object object to check for reference
+     *
+     * @param mixed $object object reference to check for reference
+     * @param mixed $objectByVal object to check for reference
      * @return Boolean true, if the reference was written, false otherwise
      */
-    protected function writeObjectReference($object){
+    protected function writeObjectReference(&$object, $objectByVal = false)
+    {
+        // Workaround for PHP5 with E_STRICT enabled complaining about "Only 
+        // variables should be passed by reference"
+        if ((null === $object) && ($objectByVal !== false)) {
+            $object = &$objectByVal;
+        }
+
         $ref = array_search($object, $this->_referenceObjects,true);
-        //quickly handle object references
-        if($ref !== false){
+
+        // quickly handle object references
+        if ($ref !== false){
             $ref <<= 1;
             $this->writeInteger($ref);
             return true;
@@ -384,7 +409,7 @@ class Zend_Amf_Parse_Amf3_Serializer extends Zend_Amf_Parse_Serializer
         if($this->writeObjectReference($object)){
             return $this;
         }
-        
+
         $className = '';
 
         //Check to see if the object is a typed object and we need to change
@@ -408,59 +433,59 @@ class Zend_Amf_Parse_Amf3_Serializer extends Zend_Amf_Parse_Serializer
                 $className = '';
                 break;
 
- 	        // By default, use object's class name
+             // By default, use object's class name
             default:
-		        $className = get_class($object);
+                $className = get_class($object);
                 break;
         }
-        
+
         $writeTraits = true;
-              
+
         //check to see, if we have a corresponding definition
         if(array_key_exists($className, $this->_referenceDefinitions)){
             $traitsInfo    = $this->_referenceDefinitions[$className]['id'];
             $encoding      = $this->_referenceDefinitions[$className]['encoding'];
             $propertyNames = $this->_referenceDefinitions[$className]['propertyNames'];
-            
+
             $traitsInfo = ($traitsInfo << 2) | 0x01;
-            
+
             $writeTraits = false;
         } else {
             $propertyNames = array();
-            
+
             if($className == ''){
                 //if there is no className, we interpret the class as dynamic without any sealed members
                 $encoding = Zend_Amf_Constants::ET_DYNAMIC;
             } else {
                 $encoding = Zend_Amf_Constants::ET_PROPLIST;
-            
+
                 foreach($object as $key => $value) {
                     if( $key[0] != "_") {
                         $propertyNames[] = $key;
                     }
                 }
             }
-                
+
             $this->_referenceDefinitions[$className] = array(
                         'id'            => count($this->_referenceDefinitions),
                         'encoding'      => $encoding,
                         'propertyNames' => $propertyNames,
                     );
-            
+
             $traitsInfo = Zend_Amf_Constants::AMF3_OBJECT_ENCODING;
             $traitsInfo |= $encoding << 2;
             $traitsInfo |= (count($propertyNames) << 4);
         }
-        
+
         $this->writeInteger($traitsInfo);
-        
+
         if($writeTraits){
             $this->writeString($className);
             foreach ($propertyNames as $value) {
                 $this->writeString($value);
             }
         }
-        
+
         try {
             switch($encoding) {
                 case Zend_Amf_Constants::ET_PROPLIST:
@@ -474,7 +499,7 @@ class Zend_Amf_Parse_Amf3_Serializer extends Zend_Amf_Parse_Serializer
                     foreach ($propertyNames as $key) {
                         $this->writeTypeMarker($object->$key);
                     }
-                    
+
                     //Write remaining properties
                     foreach($object as $key => $value){
                         if(!in_array($key,$propertyNames) && $key[0] != "_"){
@@ -482,21 +507,21 @@ class Zend_Amf_Parse_Amf3_Serializer extends Zend_Amf_Parse_Serializer
                             $this->writeTypeMarker($value);
                         }
                     }
-                    
+
                     //Write an empty string to end the dynamic part
-                    $this->writeString('');
+                    $this->writeString($this->_strEmpty);
                     break;
                 case Zend_Amf_Constants::ET_EXTERNAL:
                     require_once 'Zend/Amf/Exception.php';
                     throw new Zend_Amf_Exception('External Object Encoding not implemented');
                     break;
-                default: 
+                default:
                     require_once 'Zend/Amf/Exception.php';
                     throw new Zend_Amf_Exception('Unknown Object Encoding type: ' . $encoding);
             }
         } catch (Exception $e) {
             require_once 'Zend/Amf/Exception.php';
-            throw new Zend_Amf_Exception('Unable to writeObject output: ' . $e->getMessage());
+            throw new Zend_Amf_Exception('Unable to writeObject output: ' . $e->getMessage(), 0, $e);
         }
 
         return $this;
