@@ -14,24 +14,39 @@
  *
  * @category   Zend
  * @package    Zend_Controller
- * @copyright  Copyright (c) 2005-2008 Zend Technologies USA Inc. (http://www.zend.com)
+ * @copyright  Copyright (c) 2005-2010 Zend Technologies USA Inc. (http://www.zend.com)
  * @license    http://framework.zend.com/license/new-bsd     New BSD License
+ * @version    $Id: Action.php 22793 2010-08-05 18:32:52Z matthew $
  */
 
-/** Zend_Controller_Action_HelperBroker */
+/**
+ * @see Zend_Controller_Action_HelperBroker
+ */
 require_once 'Zend/Controller/Action/HelperBroker.php';
 
-/** Zend_Controller_Front */
+/**
+ * @see Zend_Controller_Action_Interface
+ */
+require_once 'Zend/Controller/Action/Interface.php';
+
+/**
+ * @see Zend_Controller_Front
+ */
 require_once 'Zend/Controller/Front.php';
 
 /**
  * @category   Zend
  * @package    Zend_Controller
- * @copyright  Copyright (c) 2005-2008 Zend Technologies USA Inc. (http://www.zend.com)
+ * @copyright  Copyright (c) 2005-2010 Zend Technologies USA Inc. (http://www.zend.com)
  * @license    http://framework.zend.com/license/new-bsd     New BSD License
  */
-abstract class Zend_Controller_Action
+abstract class Zend_Controller_Action implements Zend_Controller_Action_Interface
 {
+    /**
+     * @var array of existing class methods
+     */
+    protected $_classMethods;
+
     /**
      * Word delimiters (used for normalizing view script paths)
      * @var array
@@ -457,19 +472,19 @@ abstract class Zend_Controller_Action
      * overridden to implement magic (dynamic) actions, or provide run-time
      * dispatching.
      *
-     * @param string $methodName
-     * @param array $args
+     * @param  string $methodName
+     * @param  array $args
+     * @return void
+     * @throws Zend_Controller_Action_Exception
      */
     public function __call($methodName, $args)
     {
+        require_once 'Zend/Controller/Action/Exception.php';
         if ('Action' == substr($methodName, -6)) {
-            require_once 'Zend/Controller/Action/Exception.php';
             $action = substr($methodName, 0, strlen($methodName) - 6);
-            require_once 'Zend/Controller/Action/Exception.php';
             throw new Zend_Controller_Action_Exception(sprintf('Action "%s" does not exist and was not trapped in __call()', $action), 404);
         }
 
-        require_once 'Zend/Controller/Action/Exception.php';
         throw new Zend_Controller_Action_Exception(sprintf('Method "%s" does not exist and was not trapped in __call()', $methodName), 500);
     }
 
@@ -486,8 +501,12 @@ abstract class Zend_Controller_Action
 
         $this->preDispatch();
         if ($this->getRequest()->isDispatched()) {
+            if (null === $this->_classMethods) {
+                $this->_classMethods = get_class_methods($this);
+            }
+
             // preDispatch() didn't change the action, so we can continue
-            if ($this->getInvokeArg('useCaseSensitiveActions') || in_array($action, get_class_methods($this))) {
+            if ($this->getInvokeArg('useCaseSensitiveActions') || in_array($action, $this->_classMethods)) {
                 if ($this->getInvokeArg('useCaseSensitiveActions')) {
                     trigger_error('Using case sensitive actions without word separators is deprecated; please do not rely on this "feature"');
                 }
@@ -558,10 +577,10 @@ abstract class Zend_Controller_Action
      * @param mixed $default
      * @return mixed
      */
-    final protected function _getParam($paramName, $default = null)
+    protected function _getParam($paramName, $default = null)
     {
         $value = $this->getRequest()->getParam($paramName);
-        if ((null == $value) && (null !== $default)) {
+		 if ((null === $value || '' === $value) && (null !== $default)) {
             $value = $default;
         }
 
@@ -575,7 +594,7 @@ abstract class Zend_Controller_Action
      * @param mixed $value
      * @return Zend_Controller_Action
      */
-    final protected function _setParam($paramName, $value)
+    protected function _setParam($paramName, $value)
     {
         $this->getRequest()->setParam($paramName, $value);
 
@@ -589,7 +608,7 @@ abstract class Zend_Controller_Action
      * @param string $paramName
      * @return boolean
      */
-    final protected function _hasParam($paramName)
+    protected function _hasParam($paramName)
     {
         return null !== $this->getRequest()->getParam($paramName);
     }
@@ -600,7 +619,7 @@ abstract class Zend_Controller_Action
      *
      * @return array
      */
-    final protected function _getAllParams()
+    protected function _getAllParams()
     {
         return $this->getRequest()->getParams();
     }
