@@ -14,9 +14,9 @@
  *
  * @category  Zend
  * @package   Zend_Currency
- * @copyright Copyright (c) 2005-2008 Zend Technologies USA Inc. (http://www.zend.com)
+ * @copyright  Copyright (c) 2005-2010 Zend Technologies USA Inc. (http://www.zend.com)
  * @license   http://framework.zend.com/license/new-bsd     New BSD License
- * @version   $Id: Currency.php 6137 2007-08-19 14:55:27Z shreef $
+ * @version   $Id: Currency.php 22369 2010-06-04 14:54:43Z thomas $
  */
 
 /**
@@ -31,7 +31,7 @@ require_once 'Zend/Locale/Format.php';
  *
  * @category  Zend
  * @package   Zend_Currency
- * @copyright Copyright (c) 2005-2008 Zend Technologies USA Inc. (http://www.zend.com)
+ * @copyright  Copyright (c) 2005-2010 Zend Technologies USA Inc. (http://www.zend.com)
  * @license   http://framework.zend.com/license/new-bsd     New BSD License
  */
 class Zend_Currency
@@ -47,13 +47,6 @@ class Zend_Currency
     const RIGHT    = 16;
     const LEFT     = 32;
 
-    /**
-     * Locale for this currency
-     *
-     * @var string
-     */
-    private $_locale = null;
-
     /**
      * Options array
      *
@@ -66,6 +59,9 @@ class Zend_Currency
      * 'name'      => Name for this currency
      * 'currency'  => 3 lettered international abbreviation
      * 'symbol'    => Currency symbol
+     * 'locale'    => Locale for this currency
+     * 'value'     => Money value
+     * 'service'   => Exchange service to use
      *
      * @var array
      * @see Zend_Locale
@@ -78,100 +74,130 @@ class Zend_Currency
         'precision' => 2,
         'name'      => null,
         'currency'  => null,
-        'symbol'    => null
+        'symbol'    => null,
+        'locale'    => null,
+        'value'     => 0,
+        'service'   => null
     );
 
     /**
      * Creates a currency instance. Every supressed parameter is used from the actual or the given locale.
      *
-     * @param  string             $currency OPTIONAL currency short name
-     * @param  string|Zend_Locale $locale   OPTIONAL locale name
+     * @param  string|array       $options OPTIONAL Options array or currency short name
+     *                                              when string is given
+     * @param  string|Zend_Locale $locale  OPTIONAL locale name
      * @throws Zend_Currency_Exception When currency is invalid
      */
-    public function __construct($currency = null, $locale = null)
+    public function __construct($options = null, $locale = null)
     {
-        if (Zend_Locale::isLocale($currency) !== false) {
-            $temp     = $locale;
-            $locale   = $currency;
-            $currency = $temp;
+        if (is_array($options)) {
+            $this->setLocale($locale);
+            $this->setFormat($options);
+        } else if (Zend_Locale::isLocale($options, false, false)) {
+            $this->setLocale($options);
+            $options = $locale;
+        } else {
+            $this->setLocale($locale);
         }
 
-        $this->setLocale($locale);
-
         // Get currency details
-        $this->_options['currency'] = self::getShortName($currency, $this->_locale);
-        $this->_options['name']     = self::getName($currency, $this->_locale);
-        $this->_options['symbol']   = self::getSymbol($currency, $this->_locale);
+        if (!isset($this->_options['currency']) || !is_array($options)) {
+            $this->_options['currency'] = self::getShortName($options, $this->_options['locale']);
+        }
+
+        if (!isset($this->_options['name']) || !is_array($options)) {
+            $this->_options['name']     = self::getName($options, $this->_options['locale']);
+        }
+
+        if (!isset($this->_options['symbol']) || !is_array($options)) {
+            $this->_options['symbol']   = self::getSymbol($options, $this->_options['locale']);
+        }
 
         if (($this->_options['currency'] === null) and ($this->_options['name'] === null)) {
             require_once 'Zend/Currency/Exception.php';
-            throw new Zend_Currency_Exception("Currency '$currency' not found");
+            throw new Zend_Currency_Exception("Currency '$options' not found");
         }
 
         // Get the format
-        $this->_options['position'] = $this->_updateFormat();
-        $this->_options['display']  = self::NO_SYMBOL;
-        if (empty($this->_options['symbol']) === false) {
+        if (!empty($this->_options['symbol'])) {
             $this->_options['display'] = self::USE_SYMBOL;
-        } else if (empty($this->_options['currency']) === false) {
+        } else if (!empty($this->_options['currency'])) {
             $this->_options['display'] = self::USE_SHORTNAME;
         }
-
-        return $this;
-    }
-
-    /**
-     * Gets the information required for formating the currency from Zend_Locale
-     *
-     * @return Zend_Currency
-     */
-    protected function _updateFormat()
-    {
-        $locale = (empty($this->_options['format']) === true) ? $this->_locale : $this->_options['format'];
-
-        // Getting the format information of the currency
-        $format = Zend_Locale_Data::getContent($locale, 'currencynumber');
-
-        iconv_set_encoding('internal_encoding', 'UTF-8');
-        if (iconv_strpos($format, ';') !== false) {
-            $format = iconv_substr($format, 0, iconv_strpos($format, ';'));
-        }
-
-        // Knowing the sign positioning information
-        if (iconv_strpos($format, '¤') === 0) {
-            $position = self::LEFT;
-        } else if (iconv_strpos($format, '¤') === (iconv_strlen($format) - 1)) {
-            $position = self::RIGHT;
-        }
-
-        return $position;
     }
 
     /**
      * Returns a localized currency string
      *
-     * @param  integer|float $value   Currency value
+     * @param  integer|float $value   OPTIONAL Currency value
      * @param  array         $options OPTIONAL options to set temporary
      * @throws Zend_Currency_Exception When the value is not a number
      * @return string
      */
-    public function toCurrency($value, array $options = array())
+    public function toCurrency($value = null, array $options = array())
     {
+        if ($value === null) {
+            if (is_array($options) && isset($options['value'])) {
+                $value = $options['value'];
+            } else {
+                $value = $this->_options['value'];
+            }
+        }
+
+        if (is_array($value)) {
+            $options += $value;
+            if (isset($options['value'])) {
+                $value = $options['value'];
+            }
+        }
+
         // Validate the passed number
-        if ((isset($value) === false) or (is_numeric($value) === false)) {
+        if (!(isset($value)) or (is_numeric($value) === false)) {
             require_once 'Zend/Currency/Exception.php';
             throw new Zend_Currency_Exception("Value '$value' has to be numeric");
         }
 
+        if (isset($options['currency'])) {
+            if (!isset($options['locale'])) {
+                $options['locale'] = $this->_options['locale'];
+            }
+
+            $options['currency'] = self::getShortName($options['currency'], $options['locale']);
+            $options['name']     = self::getName($options['currency'], $options['locale']);
+            $options['symbol']   = self::getSymbol($options['currency'], $options['locale']);
+        }
+
         $options = $this->_checkOptions($options) + $this->_options;
 
         // Format the number
-        if (empty($options['format']) === true) {
-            $options['format'] = $this->_locale;
+        $format = $options['format'];
+        $locale = $options['locale'];
+        if (empty($format)) {
+            $format = Zend_Locale_Data::getContent($locale, 'currencynumber');
+        } else if (Zend_Locale::isLocale($format, true, false)) {
+            $locale = $format;
+            $format = Zend_Locale_Data::getContent($format, 'currencynumber');
         }
 
-        $value = Zend_Locale_Format::toNumber($value, array('locale' => $options['format'],
-                                                            'precision' => $options['precision']));
+        $original = $value;
+        $value    = Zend_Locale_Format::toNumber($value, array('locale'        => $locale,
+                                                               'number_format' => $format,
+                                                               'precision'     => $options['precision']));
+
+        if ($options['position'] !== self::STANDARD) {
+            $value = str_replace('¤', '', $value);
+            $space = '';
+            if (iconv_strpos($value, ' ') !== false) {
+                $value = str_replace(' ', '', $value);
+                $space = ' ';
+            }
+
+            if ($options['position'] == self::LEFT) {
+                $value = '¤' . $space . $value;
+            } else {
+                $value = $value . $space . '¤';
+            }
+        }
 
         // Localize the number digits
         if (empty($options['script']) === false) {
@@ -180,35 +206,68 @@ class Zend_Currency
 
         // Get the sign to be placed next to the number
         if (is_numeric($options['display']) === false) {
-            $sign = ' ' . $options['display'] . ' ';
+            $sign = $options['display'];
         } else {
             switch($options['display']) {
                 case self::USE_SYMBOL:
-                    $sign = ' ' . $options['symbol'] . ' ';
+                    $sign = $this->_extractPattern($options['symbol'], $original);
                     break;
 
                 case self::USE_SHORTNAME:
-                    $sign = ' ' . $options['currency'] . ' ';
+                    $sign = $options['currency'];
                     break;
 
                 case self::USE_NAME:
-                    $sign = ' ' . $options['name'] . ' ';
+                    $sign = $options['name'];
                     break;
 
                 default:
                     $sign = '';
+                    $value = str_replace(' ', '', $value);
                     break;
             }
         }
 
-        // Place the sign next to the number
-        if ($options['position'] === self::RIGHT) {
-            $value = $value . $sign;
-        } else if ($options['position'] === self::LEFT) {
-            $value = $sign . $value;
+        $value = str_replace('¤', $sign, $value);
+        return $value;
+    }
+
+    /**
+     * Internal method to extract the currency pattern
+     * when a choice is given based on the given value
+     *
+     * @param  string $pattern
+     * @param  float|integer $value
+     * @return string
+     */
+    private function _extractPattern($pattern, $value)
+    {
+        if (strpos($pattern, '|') === false) {
+            return $pattern;
         }
 
-        return trim($value);
+        $patterns = explode('|', $pattern);
+        $token    = $pattern;
+        $value    = trim(str_replace('¤', '', $value));
+        krsort($patterns);
+        foreach($patterns as $content) {
+            if (strpos($content, '<') !== false) {
+                $check = iconv_substr($content, 0, iconv_strpos($content, '<'));
+                $token = iconv_substr($content, iconv_strpos($content, '<') + 1);
+                if ($check < $value) {
+                    return $token;
+                }
+            } else {
+                $check = iconv_substr($content, 0, iconv_strpos($content, '≤'));
+                $token = iconv_substr($content, iconv_strpos($content, '≤') + 1);
+                if ($check <= $value) {
+                    return $token;
+                }
+            }
+
+        }
+
+        return $token;
     }
 
     /**
@@ -236,24 +295,19 @@ class Zend_Currency
     private function _checkParams($currency = null, $locale = null)
     {
         // Manage the params
-        if ((empty($locale) === true) and (empty($currency) === false) and
-            (Zend_Locale::isLocale($currency) !== false)) {
+        if ((empty($locale)) and (!empty($currency)) and
+            (Zend_Locale::isLocale($currency, true, false))) {
             $locale   = $currency;
             $currency = null;
         }
 
-        if ($locale instanceof Zend_Locale) {
-            $locale = $locale->toString();
-        }
-
         // Validate the locale and get the country short name
         $country = null;
-        $locale  = Zend_Locale::isLocale($locale);
-        if (($locale !== false) and (strlen($locale) > 4)) {
+        if ((Zend_Locale::isLocale($locale, true, false)) and (strlen($locale) > 4)) {
             $country = substr($locale, (strpos($locale, '_') + 1));
         } else {
             require_once 'Zend/Currency/Exception.php';
-            throw new Zend_Currency_Exception("No region found within the locale '$locale'");
+            throw new Zend_Currency_Exception("No region found within the locale '" . (string) $locale . "'");
         }
 
         // Get the available currencies for this country
@@ -394,8 +448,8 @@ class Zend_Currency
     public function getCurrencyList($region = null)
     {
         if (empty($region) === true) {
-            if (strlen($this->_locale) > 4) {
-                $region = substr($this->_locale, (strpos($this->_locale, '_') + 1));
+            if (strlen($this->_options['locale']) > 4) {
+                $region = substr($this->_options['locale'], (strpos($this->_options['locale'], '_') + 1));
             }
         }
 
@@ -409,7 +463,7 @@ class Zend_Currency
      */
     public function toString()
     {
-        return (empty($this->_options['name']) === false) ? $this->_options['name'] : $this->_options['currency'];
+        return $this->toCurrency();
     }
 
     /**
@@ -422,17 +476,6 @@ class Zend_Currency
         return $this->toString();
     }
 
-    /**
-     * Sets a cache for Zend_Currency
-     *
-     * @param  Zend_Cache_Core $cache Cache to set
-     * @return void
-     */
-    public static function setCache(Zend_Cache_Core $cache)
-    {
-        Zend_Locale_Data::setCache($cache);
-    }
-
     /**
      * Returns the set cache
      *
@@ -444,34 +487,78 @@ class Zend_Currency
         return $cache;
     }
 
+    /**
+     * Sets a cache for Zend_Currency
+     *
+     * @param  Zend_Cache_Core $cache Cache to set
+     * @return void
+     */
+    public static function setCache(Zend_Cache_Core $cache)
+    {
+        Zend_Locale_Data::setCache($cache);
+    }
+
+    /**
+     * Returns true when a cache is set
+     *
+     * @return boolean
+     */
+    public static function hasCache()
+    {
+        return Zend_Locale_Data::hasCache();
+    }
+
+    /**
+     * Removes any set cache
+     *
+     * @return void
+     */
+    public static function removeCache()
+    {
+        Zend_Locale_Data::removeCache();
+    }
+
+    /**
+     * Clears all set cache data
+     *
+     * @return void
+     */
+    public static function clearCache()
+    {
+        Zend_Locale_Data::clearCache();
+    }
+
     /**
      * Sets a new locale for data retreivement
-     * Returned is the really set locale
      * Example: 'de_XX' will be set to 'de' because 'de_XX' does not exist
      * 'xx_YY' will be set to 'root' because 'xx' does not exist
      *
      * @param  string|Zend_Locale $locale (Optional) Locale for parsing input
      * @throws Zend_Currency_Exception When the given locale does not exist
-     * @return string
+     * @return Zend_Currency Provides fluent interface
      */
     public function setLocale($locale = null)
     {
-        if ($locale instanceof Zend_Locale) {
-            $this->_locale = $locale->toString();
-        } else {
-            $this->_locale = Zend_Locale::isLocale($locale, true);
-            if ($this->_locale === false) {
+        require_once 'Zend/Locale.php';
+        try {
+            $locale = Zend_Locale::findLocale($locale);
+            if (strlen($locale) > 4) {
+                $this->_options['locale'] = $locale;
+            } else {
                 require_once 'Zend/Currency/Exception.php';
-                throw new Zend_Currency_Exception("Given locale ($locale) does not exist");
+                throw new Zend_Currency_Exception("No region found within the locale '" . (string) $locale . "'");
             }
+        } catch (Zend_Locale_Exception $e) {
+            require_once 'Zend/Currency/Exception.php';
+            throw new Zend_Currency_Exception($e->getMessage());
         }
 
         // Get currency details
-        $this->_options['currency'] = $this->getShortName(null, $this->_locale);
-        $this->_options['name']     = $this->getName(null, $this->_locale);
-        $this->_options['symbol']   = $this->getSymbol(null, $this->_locale);
+        $this->_options['currency'] = $this->getShortName(null, $this->_options['locale']);
+        $this->_options['name']     = $this->getName(null, $this->_options['locale']);
+        $this->_options['symbol']   = $this->getSymbol(null, $this->_options['locale']);
 
-        return $this->getLocale();
+        return $this;
     }
 
     /**
@@ -481,7 +568,242 @@ class Zend_Currency
      */
     public function getLocale()
     {
-        return $this->_locale;
+        return $this->_options['locale'];
+    }
+
+    /**
+     * Returns the value
+     *
+     * @return float
+     */
+    public function getValue()
+    {
+        return $this->_options['value'];
+    }
+
+    /**
+     * Adds a currency
+     *
+     * @param float|integer|Zend_Currency $value    Add this value to currency
+     * @param string|Zend_Currency        $currency The currency to add
+     * @return Zend_Currency
+     */
+    public function setValue($value, $currency = null)
+    {
+        $this->_options['value'] = $this->_exchangeCurrency($value, $currency);
+        return $this;
+    }
+
+    /**
+     * Adds a currency
+     *
+     * @param float|integer|Zend_Currency $value    Add this value to currency
+     * @param string|Zend_Currency        $currency The currency to add
+     * @return Zend_Currency
+     */
+    public function add($value, $currency = null)
+    {
+        $value = $this->_exchangeCurrency($value, $currency);
+        $this->_options['value'] += (float) $value;
+        return $this;
+    }
+
+    /**
+     * Substracts a currency
+     *
+     * @param float|integer|Zend_Currency $value    Substracts this value from currency
+     * @param string|Zend_Currency        $currency The currency to substract
+     * @return Zend_Currency
+     */
+    public function sub($value, $currency = null)
+    {
+        $value = $this->_exchangeCurrency($value, $currency);
+        $this->_options['value'] -= (float) $value;
+        return $this;
+    }
+
+    /**
+     * Divides a currency
+     *
+     * @param float|integer|Zend_Currency $value    Divides this value from currency
+     * @param string|Zend_Currency        $currency The currency to divide
+     * @return Zend_Currency
+     */
+    public function div($value, $currency = null)
+    {
+        $value = $this->_exchangeCurrency($value, $currency);
+        $this->_options['value'] /= (float) $value;
+        return $this;
+    }
+
+    /**
+     * Multiplies a currency
+     *
+     * @param float|integer|Zend_Currency $value    Multiplies this value from currency
+     * @param string|Zend_Currency        $currency The currency to multiply
+     * @return Zend_Currency
+     */
+    public function mul($value, $currency = null)
+    {
+        $value = $this->_exchangeCurrency($value, $currency);
+        $this->_options['value'] *= (float) $value;
+        return $this;
+    }
+
+    /**
+     * Calculates the modulo from a currency
+     *
+     * @param float|integer|Zend_Currency $value    Calculate modulo from this value
+     * @param string|Zend_Currency        $currency The currency to calculate the modulo
+     * @return Zend_Currency
+     */
+    public function mod($value, $currency = null)
+    {
+        $value = $this->_exchangeCurrency($value, $currency);
+        $this->_options['value'] %= (float) $value;
+        return $this;
+    }
+
+    /**
+     * Compares two currencies
+     *
+     * @param float|integer|Zend_Currency $value    Compares the currency with this value
+     * @param string|Zend_Currency        $currency The currency to compare this value from
+     * @return Zend_Currency
+     */
+    public function compare($value, $currency = null)
+    {
+        $value = $this->_exchangeCurrency($value, $currency);
+        $value = $this->_options['value'] - $value;
+        if ($value < 0) {
+            return -1;
+        } else if ($value > 0) {
+            return 1;
+        }
+
+        return 0;
+    }
+
+    /**
+     * Returns true when the two currencies are equal
+     *
+     * @param float|integer|Zend_Currency $value    Compares the currency with this value
+     * @param string|Zend_Currency        $currency The currency to compare this value from
+     * @return boolean
+     */
+    public function equals($value, $currency = null)
+    {
+        $value = $this->_exchangeCurrency($value, $currency);
+        if ($this->_options['value'] == $value) {
+            return true;
+        }
+
+        return false;
+    }
+
+    /**
+     * Returns true when the currency is more than the given value
+     *
+     * @param float|integer|Zend_Currency $value    Compares the currency with this value
+     * @param string|Zend_Currency        $currency The currency to compare this value from
+     * @return boolean
+     */
+    public function isMore($value, $currency = null)
+    {
+        $value = $this->_exchangeCurrency($value, $currency);
+        if ($this->_options['value'] > $value) {
+            return true;
+        }
+
+        return false;
+    }
+
+    /**
+     * Returns true when the currency is less than the given value
+     *
+     * @param float|integer|Zend_Currency $value    Compares the currency with this value
+     * @param string|Zend_Currency        $currency The currency to compare this value from
+     * @return boolean
+     */
+    public function isLess($value, $currency = null)
+    {
+        $value = $this->_exchangeCurrency($value, $currency);
+        if ($this->_options['value'] < $value) {
+            return true;
+        }
+
+        return false;
+
+    }
+
+    /**
+     * Internal method which calculates the exchanges currency
+     *
+     * @param float|integer|Zend_Currency $value    Compares the currency with this value
+     * @param string|Zend_Currency        $currency The currency to compare this value from
+     * @return unknown
+     */
+    protected function _exchangeCurrency($value, $currency)
+    {
+        if ($value instanceof Zend_Currency) {
+            $currency = $value->getShortName();
+            $value    = $value->getValue();
+        } else {
+            $currency = $this->getShortName($currency, $this->getLocale());
+        }
+
+        $rate = 1;
+        if ($currency !== $this->getShortName()) {
+            $service = $this->getService();
+            if (!($service instanceof Zend_Currency_CurrencyInterface)) {
+                require_once 'Zend/Currency/Exception.php';
+                throw new Zend_Currency_Exception('No exchange service applied');
+            }
+
+            $rate = $service->getRate($currency, $this->getShortName());
+        }
+
+        $value *= $rate;
+        return $value;
+    }
+
+    /**
+     * Returns the set service class
+     *
+     * @return Zend_Service
+     */
+    public function getService()
+    {
+        return $this->_options['service'];
+    }
+
+    /**
+     * Sets a new exchange service
+     *
+     * @param string|Zend_Currency_CurrencyInterface $service Service class
+     * @return Zend_Currency
+     */
+    public function setService($service)
+    {
+        if (is_string($service)) {
+            require_once 'Zend/Loader.php';
+            if (!class_exists($service)) {
+                $file = str_replace('_', DIRECTORY_SEPARATOR, $service) . '.php';
+                if (Zend_Loader::isReadable($file)) {
+                    Zend_Loader::loadClass($class);
+                }
+            }
+
+            $service = new $service;
+        }
+
+        if (!($service instanceof Zend_Currency_CurrencyInterface)) {
+            require_once 'Zend/Currency/Exception.php';
+            throw new Zend_Currency_Exception('A currency service must implement Zend_Currency_CurrencyInterface');
+        }
+
+        $this->_options['service'] = $service;
+        return $this;
     }
 
     /**
@@ -496,7 +818,7 @@ class Zend_Currency
      * @throws Zend_Currency_Exception On unknown options
      * @return array
      */
-    private function _checkOptions(array $options = array())
+    protected function _checkOptions(array $options = array())
     {
         if (count($options) === 0) {
             return $this->_options;
@@ -517,17 +839,16 @@ class Zend_Currency
                         throw new Zend_Currency_Exception("Unknown position '" . $value . "'");
                     }
 
-                    if ($value === self::STANDARD) {
-                        $options['position'] = $this->_updateFormat();
-                    }
                     break;
 
                 case 'format':
-                    if ((empty($value) === false) and (Zend_Locale::isLocale($value) === false)) {
-                        require_once 'Zend/Currency/Exception.php';
-                        throw new Zend_Currency_Exception("'" .
-                            ((gettype($value) === 'object') ? get_class($value) : $value)
-                            . "' is not a known locale.");
+                    if ((empty($value) === false) and (Zend_Locale::isLocale($value, null, false) === false)) {
+                        if (!is_string($value) || (strpos($value, '0') === false)) {
+                            require_once 'Zend/Currency/Exception.php';
+                            throw new Zend_Currency_Exception("'" .
+                                ((gettype($value) === 'object') ? get_class($value) : $value)
+                                . "' is no format token");
+                        }
                     }
                     break;
 
@@ -559,17 +880,7 @@ class Zend_Currency
                     }
                     break;
 
-                case 'name':
-                    // Break intentionally omitted
-                case 'currency':
-                    // Break intentionally omitted
-                case 'symbol':
-                    // Unchecked options
-                    break;
-
                 default:
-                    require_once 'Zend/Currency/Exception.php';
-                    throw new Zend_Currency_Exception("Unknown option: '$name' = '$value'");
                     break;
             }
         }
