@@ -15,9 +15,9 @@
  * @category   Zend
  * @package    Zend_Db
  * @subpackage Table
- * @copyright  Copyright (c) 2005-2008 Zend Technologies USA Inc. (http://www.zend.com)
+ * @copyright  Copyright (c) 2005-2010 Zend Technologies USA Inc. (http://www.zend.com)
  * @license    http://framework.zend.com/license/new-bsd     New BSD License
- * @version    $Id: Abstract.php 6320 2007-09-12 00:27:22Z bkarwin $
+ * @version    $Id: Abstract.php 21079 2010-02-18 18:15:49Z tech13 $
  */
 
 /**
@@ -41,19 +41,22 @@ require_once 'Zend/Db.php';
  * @category   Zend
  * @package    Zend_Db
  * @subpackage Table
- * @copyright  Copyright (c) 2005-2008 Zend Technologies USA Inc. (http://www.zend.com)
+ * @copyright  Copyright (c) 2005-2010 Zend Technologies USA Inc. (http://www.zend.com)
  * @license    http://framework.zend.com/license/new-bsd     New BSD License
  */
 abstract class Zend_Db_Table_Abstract
 {
 
     const ADAPTER          = 'db';
+    const DEFINITION        = 'definition';
+    const DEFINITION_CONFIG_NAME = 'definitionConfigName';
     const SCHEMA           = 'schema';
     const NAME             = 'name';
     const PRIMARY          = 'primary';
     const COLS             = 'cols';
     const METADATA         = 'metadata';
     const METADATA_CACHE   = 'metadataCache';
+    const METADATA_CACHE_IN_CLASS = 'metadataCacheInClass';
     const ROW_CLASS        = 'rowClass';
     const ROWSET_CLASS     = 'rowsetClass';
     const REFERENCE_MAP    = 'referenceMap';
@@ -70,6 +73,13 @@ abstract class Zend_Db_Table_Abstract
     const RESTRICT         = 'restrict';
     const SET_NULL         = 'setNull';
 
+    const DEFAULT_NONE     = 'defaultNone';
+    const DEFAULT_CLASS    = 'defaultClass';
+    const DEFAULT_DB       = 'defaultDb';
+
+    const SELECT_WITH_FROM_PART    = true;
+    const SELECT_WITHOUT_FROM_PART = false;
+
     /**
      * Default Zend_Db_Adapter_Abstract object.
      *
@@ -77,6 +87,20 @@ abstract class Zend_Db_Table_Abstract
      */
     protected static $_defaultDb;
 
+    /**
+     * Optional Zend_Db_Table_Definition object
+     *
+     * @var unknown_type
+     */
+    protected $_definition = null;
+
+    /**
+     * Optional definition config name used in concrete implementation
+     *
+     * @var string
+     */
+    protected $_definitionConfigName = null;
+
     /**
      * Default cache for information provided by the adapter's describeTable() method.
      *
@@ -101,7 +125,7 @@ abstract class Zend_Db_Table_Abstract
     /**
      * The table name.
      *
-     * @var array
+     * @var string
      */
     protected $_name = null;
 
@@ -156,6 +180,12 @@ abstract class Zend_Db_Table_Abstract
      */
     protected $_metadataCache = null;
 
+    /**
+     * Flag: whether or not to cache metadata in the class
+     * @var bool
+     */
+    protected $_metadataCacheInClass = true;
+
     /**
      * Classname for row
      *
@@ -200,6 +230,10 @@ abstract class Zend_Db_Table_Abstract
      */
     protected $_dependentTables = array();
 
+
+    protected $_defaultSource = self::DEFAULT_NONE;
+    protected $_defaultValues = array();
+
     /**
      * Constructor.
      *
@@ -227,11 +261,33 @@ abstract class Zend_Db_Table_Abstract
             $config = array(self::ADAPTER => $config);
         }
 
-        foreach ($config as $key => $value) {
+        if ($config) {
+            $this->setOptions($config);
+        }
+
+        $this->_setup();
+        $this->init();
+    }
+
+    /**
+     * setOptions()
+     *
+     * @param array $options
+     * @return Zend_Db_Table_Abstract
+     */
+    public function setOptions(Array $options)
+    {
+        foreach ($options as $key => $value) {
             switch ($key) {
                 case self::ADAPTER:
                     $this->_setAdapter($value);
                     break;
+                case self::DEFINITION:
+                    $this->setDefinition($value);
+                    break;
+                case self::DEFINITION_CONFIG_NAME:
+                    $this->setDefinitionConfigName($value);
+                    break;
                 case self::SCHEMA:
                     $this->_schema = (string) $value;
                     break;
@@ -256,6 +312,9 @@ abstract class Zend_Db_Table_Abstract
                 case self::METADATA_CACHE:
                     $this->_setMetadataCache($value);
                     break;
+                case self::METADATA_CACHE_IN_CLASS:
+                    $this->setMetadataCacheInClass($value);
+                    break;
                 case self::SEQUENCE:
                     $this->_setSequence($value);
                     break;
@@ -265,8 +324,51 @@ abstract class Zend_Db_Table_Abstract
             }
         }
 
-        $this->_setup();
-        $this->init();
+        return $this;
+    }
+
+    /**
+     * setDefinition()
+     *
+     * @param Zend_Db_Table_Definition $definition
+     * @return Zend_Db_Table_Abstract
+     */
+    public function setDefinition(Zend_Db_Table_Definition $definition)
+    {
+        $this->_definition = $definition;
+        return $this;
+    }
+
+    /**
+     * getDefinition()
+     *
+     * @return Zend_Db_Table_Definition|null
+     */
+    public function getDefinition()
+    {
+        return $this->_definition;
+    }
+
+    /**
+     * setDefinitionConfigName()
+     *
+     * @param string $definition
+     * @return Zend_Db_Table_Abstract
+     */
+    public function setDefinitionConfigName($definitionConfigName)
+    {
+        $this->_definitionConfigName = $definitionConfigName;
+        return $this;
+    }
+
+    /**
+     * getDefinitionConfigName()
+     *
+     * @return string
+     */
+    public function getDefinitionConfigName()
+    {
+        return $this->_definitionConfigName;
     }
 
     /**
@@ -307,6 +409,37 @@ abstract class Zend_Db_Table_Abstract
         return $this->_rowsetClass;
     }
 
+    /**
+     * Add a reference to the reference map
+     *
+     * @param string $ruleKey
+     * @param string|array $columns
+     * @param string $refTableClass
+     * @param string|array $refColumns
+     * @param string $onDelete
+     * @param string $onUpdate
+     * @return Zend_Db_Table_Abstract
+     */
+    public function addReference($ruleKey, $columns, $refTableClass, $refColumns,
+                                 $onDelete = null, $onUpdate = null)
+    {
+        $reference = array(self::COLUMNS         => (array) $columns,
+                           self::REF_TABLE_CLASS => $refTableClass,
+                           self::REF_COLUMNS     => (array) $refColumns);
+
+        if (!empty($onDelete)) {
+            $reference[self::ON_DELETE] = $onDelete;
+        }
+
+        if (!empty($onUpdate)) {
+            $reference[self::ON_UPDATE] = $onUpdate;
+        }
+
+        $this->_referenceMap[$ruleKey] = $reference;
+
+        return $this;
+    }
+
     /**
      * @param array $referenceMap
      * @return Zend_Db_Table_Abstract Provides a fluent interface
@@ -327,6 +460,9 @@ abstract class Zend_Db_Table_Abstract
     public function getReference($tableClassname, $ruleKey = null)
     {
         $thisClass = get_class($this);
+        if ($thisClass === 'Zend_Db_Table') {
+            $thisClass = $this->_definitionConfigName;
+        }
         $refMap = $this->_getReferenceMapNormalized();
         if ($ruleKey !== null) {
             if (!isset($refMap[$ruleKey])) {
@@ -367,13 +503,61 @@ abstract class Zend_Db_Table_Abstract
         return $this->_dependentTables;
     }
 
+    /**
+     * set the defaultSource property - this tells the table class where to find default values
+     *
+     * @param string $defaultSource
+     * @return Zend_Db_Table_Abstract
+     */
+    public function setDefaultSource($defaultSource = self::DEFAULT_NONE)
+    {
+        if (!in_array($defaultSource, array(self::DEFAULT_CLASS, self::DEFAULT_DB, self::DEFAULT_NONE))) {
+            $defaultSource = self::DEFAULT_NONE;
+        }
+
+        $this->_defaultSource = $defaultSource;
+        return $this;
+    }
+
+    /**
+     * returns the default source flag that determines where defaultSources come from
+     *
+     * @return unknown
+     */
+    public function getDefaultSource()
+    {
+        return $this->_defaultSource;
+    }
+
+    /**
+     * set the default values for the table class
+     *
+     * @param array $defaultValues
+     * @return Zend_Db_Table_Abstract
+     */
+    public function setDefaultValues(Array $defaultValues)
+    {
+        foreach ($defaultValues as $defaultName => $defaultValue) {
+            if (array_key_exists($defaultName, $this->_metadata)) {
+                $this->_defaultValues[$defaultName] = $defaultValue;
+            }
+        }
+        return $this;
+    }
+
+    public function getDefaultValues()
+    {
+        return $this->_defaultValues;
+    }
+
+
     /**
      * Sets the default Zend_Db_Adapter_Abstract for all Zend_Db_Table objects.
      *
      * @param  mixed $db Either an Adapter object, or a string naming a Registry key
      * @return void
      */
-    public static final function setDefaultAdapter($db = null)
+    public static function setDefaultAdapter($db = null)
     {
         self::$_defaultDb = self::_setupAdapter($db);
     }
@@ -383,7 +567,7 @@ abstract class Zend_Db_Table_Abstract
      *
      * @return Zend_Db_Adapter_Abstract or null
      */
-    public static final function getDefaultAdapter()
+    public static function getDefaultAdapter()
     {
         return self::$_defaultDb;
     }
@@ -392,7 +576,7 @@ abstract class Zend_Db_Table_Abstract
      * @param  mixed $db Either an Adapter object, or a string naming a Registry key
      * @return Zend_Db_Table_Abstract Provides a fluent interface
      */
-    protected final function _setAdapter($db)
+    protected function _setAdapter($db)
     {
         $this->_db = self::_setupAdapter($db);
         return $this;
@@ -403,7 +587,7 @@ abstract class Zend_Db_Table_Abstract
      *
      * @return Zend_Db_Adapter_Abstract
      */
-    public final function getAdapter()
+    public function getAdapter()
     {
         return $this->_db;
     }
@@ -413,7 +597,7 @@ abstract class Zend_Db_Table_Abstract
      * @return Zend_Db_Adapter_Abstract
      * @throws Zend_Db_Table_Exception
      */
-    protected static final function _setupAdapter($db)
+    protected static function _setupAdapter($db)
     {
         if ($db === null) {
             return null;
@@ -479,12 +663,36 @@ abstract class Zend_Db_Table_Abstract
         return $this->_metadataCache;
     }
 
+    /**
+     * Indicate whether metadata should be cached in the class for the duration
+     * of the instance
+     *
+     * @param  bool $flag
+     * @return Zend_Db_Table_Abstract
+     */
+    public function setMetadataCacheInClass($flag)
+    {
+        $this->_metadataCacheInClass = (bool) $flag;
+        return $this;
+    }
+
+    /**
+     * Retrieve flag indicating if metadata should be cached for duration of
+     * instance
+     *
+     * @return bool
+     */
+    public function metadataCacheInClass()
+    {
+        return $this->_metadataCacheInClass;
+    }
+
     /**
      * @param mixed $metadataCache Either a Cache object, or a string naming a Registry key
      * @return Zend_Cache_Core
      * @throws Zend_Db_Table_Exception
      */
-    protected static final function _setupMetadataCache($metadataCache)
+    protected static function _setupMetadataCache($metadataCache)
     {
         if ($metadataCache === null) {
             return null;
@@ -530,8 +738,6 @@ abstract class Zend_Db_Table_Abstract
     {
         $this->_setupDatabaseAdapter();
         $this->_setupTableName();
-        $this->_setupMetadata();
-        $this->_setupPrimaryKey();
     }
 
     /**
@@ -581,6 +787,10 @@ abstract class Zend_Db_Table_Abstract
      */
     protected function _setupMetadata()
     {
+        if ($this->metadataCacheInClass() && (count($this->_metadata) > 0)) {
+            return true;
+        }
+
         // Assume that metadata will be loaded from cache
         $isMetadataFromCache = true;
 
@@ -593,7 +803,16 @@ abstract class Zend_Db_Table_Abstract
         // If $this has a metadata cache
         if (null !== $this->_metadataCache) {
             // Define the cache identifier where the metadata are saved
-            $cacheId = md5("$this->_schema.$this->_name");
+
+            //get db configuration
+            $dbConfig = $this->_db->getConfig();
+
+            // Define the cache identifier where the metadata are saved
+            $cacheId = md5( // port:host/dbname:schema.table (based on availabilty)
+                (isset($dbConfig['options']['port']) ? ':'.$dbConfig['options']['port'] : null)
+                . (isset($dbConfig['options']['host']) ? ':'.$dbConfig['options']['host'] : null)
+                . '/'.$dbConfig['dbname'].':'.$this->_schema.'.'.$this->_name
+                );
         }
 
         // If $this has no metadata cache or metadata cache misses
@@ -604,24 +823,31 @@ abstract class Zend_Db_Table_Abstract
             $metadata = $this->_db->describeTable($this->_name, $this->_schema);
             // If $this has a metadata cache, then cache the metadata
             if (null !== $this->_metadataCache && !$this->_metadataCache->save($metadata, $cacheId)) {
-                /**
-                 * @see Zend_Db_Table_Exception
-                 */
-                require_once 'Zend/Db/Table/Exception.php';
-                throw new Zend_Db_Table_Exception('Failed saving metadata to metadataCache');
+                trigger_error('Failed saving metadata to metadataCache', E_USER_NOTICE);
             }
         }
 
         // Assign the metadata to $this
         $this->_metadata = $metadata;
 
-        // Update the columns
-        $this->_cols = array_keys($this->_metadata);
-
         // Return whether the metadata were loaded from cache
         return $isMetadataFromCache;
     }
 
+    /**
+     * Retrieve table columns
+     *
+     * @return array
+     */
+    protected function _getCols()
+    {
+        if (null === $this->_cols) {
+            $this->_setupMetadata();
+            $this->_cols = array_keys($this->_metadata);
+        }
+        return $this->_cols;
+    }
+
     /**
      * Initialize primary key from metadata.
      * If $_primary is not defined, discover primary keys
@@ -633,6 +859,7 @@ abstract class Zend_Db_Table_Abstract
     protected function _setupPrimaryKey()
     {
         if (!$this->_primary) {
+            $this->_setupMetadata();
             $this->_primary = array();
             foreach ($this->_metadata as $col) {
                 if ($col['PRIMARY']) {
@@ -655,12 +882,13 @@ abstract class Zend_Db_Table_Abstract
             unset($this->_primary[0]);
         }
 
-        if (! array_intersect((array) $this->_primary, $this->_cols) == (array) $this->_primary) {
+        $cols = $this->_getCols();
+        if (! array_intersect((array) $this->_primary, $cols) == (array) $this->_primary) {
             require_once 'Zend/Db/Table/Exception.php';
             throw new Zend_Db_Table_Exception("Primary key column(s) ("
                 . implode(',', (array) $this->_primary)
                 . ") are not columns in this table ("
-                . implode(',', $this->_cols)
+                . implode(',', $cols)
                 . ")");
         }
 
@@ -672,9 +900,9 @@ abstract class Zend_Db_Table_Abstract
          * object whose name is "<table>_<column>_seq".
          */
         if ($this->_sequence === true && $this->_db instanceof Zend_Db_Adapter_Pdo_Pgsql) {
-            $this->_sequence = "{$this->_name}_{$pkIdentity}_seq";
+            $this->_sequence = $this->_db->quoteIdentifier("{$this->_name}_{$pkIdentity}_seq");
             if ($this->_schema) {
-                $this->_sequence = $this->_schema . '.' . $this->_sequence;
+                $this->_sequence = $this->_db->quoteIdentifier($this->_schema) . '.' . $this->_sequence;
             }
         }
     }
@@ -738,40 +966,47 @@ abstract class Zend_Db_Table_Abstract
      */
     public function info($key = null)
     {
+        $this->_setupPrimaryKey();
+
         $info = array(
             self::SCHEMA           => $this->_schema,
             self::NAME             => $this->_name,
-            self::COLS             => (array) $this->_cols,
+            self::COLS             => $this->_getCols(),
             self::PRIMARY          => (array) $this->_primary,
             self::METADATA         => $this->_metadata,
-            self::ROW_CLASS        => $this->_rowClass,
-            self::ROWSET_CLASS     => $this->_rowsetClass,
+            self::ROW_CLASS        => $this->getRowClass(),
+            self::ROWSET_CLASS     => $this->getRowsetClass(),
             self::REFERENCE_MAP    => $this->_referenceMap,
             self::DEPENDENT_TABLES => $this->_dependentTables,
             self::SEQUENCE         => $this->_sequence
         );
-        
+
         if ($key === null) {
             return $info;
         }
-        
+
         if (!array_key_exists($key, $info)) {
             require_once 'Zend/Db/Table/Exception.php';
             throw new Zend_Db_Table_Exception('There is no table information for the key "' . $key . '"');
         }
-        
+
         return $info[$key];
     }
 
     /**
      * Returns an instance of a Zend_Db_Table_Select object.
      *
+     * @param bool $withFromPart Whether or not to include the from part of the select based on the table
      * @return Zend_Db_Table_Select
      */
-    public function select()
+    public function select($withFromPart = self::SELECT_WITHOUT_FROM_PART)
     {
         require_once 'Zend/Db/Table/Select.php';
-        return new Zend_Db_Table_Select($this);
+        $select = new Zend_Db_Table_Select($this);
+        if ($withFromPart == self::SELECT_WITH_FROM_PART) {
+            $select->from($this->info(self::NAME), Zend_Db_Table_Select::SQL_WILDCARD, $this->info(self::SCHEMA));
+        }
+        return $select;
     }
 
     /**
@@ -782,6 +1017,8 @@ abstract class Zend_Db_Table_Abstract
      */
     public function insert(array $data)
     {
+        $this->_setupPrimaryKey();
+
         /**
          * Zend_Db_Table assumes that if you have a compound primary key
          * and one of the columns in the key uses a sequence,
@@ -836,6 +1073,29 @@ abstract class Zend_Db_Table_Abstract
         return $pkData;
     }
 
+    /**
+     * Check if the provided column is an identity of the table
+     *
+     * @param  string $column
+     * @throws Zend_Db_Table_Exception
+     * @return boolean
+     */
+    public function isIdentity($column)
+    {
+        $this->_setupPrimaryKey();
+
+        if (!isset($this->_metadata[$column])) {
+            /**
+             * @see Zend_Db_Table_Exception
+             */
+            require_once 'Zend/Db/Table/Exception.php';
+
+            throw new Zend_Db_Table_Exception('Column "' . $column . '" not found in table.');
+        }
+
+        return (bool) $this->_metadata[$column]['IDENTITY'];
+    }
+
     /**
      * Updates existing rows.
      *
@@ -859,6 +1119,7 @@ abstract class Zend_Db_Table_Abstract
      */
     public function _cascadeUpdate($parentTableClassname, array $oldPrimaryKey, array $newPrimaryKey)
     {
+        $this->_setupMetadata();
         $rowsAffected = 0;
         foreach ($this->_getReferenceMapNormalized() as $map) {
             if ($map[self::REF_TABLE_CLASS] == $parentTableClassname && isset($map[self::ON_UPDATE])) {
@@ -909,12 +1170,13 @@ abstract class Zend_Db_Table_Abstract
      */
     public function _cascadeDelete($parentTableClassname, array $primaryKey)
     {
+        $this->_setupMetadata();
         $rowsAffected = 0;
         foreach ($this->_getReferenceMapNormalized() as $map) {
             if ($map[self::REF_TABLE_CLASS] == $parentTableClassname && isset($map[self::ON_DELETE])) {
                 switch ($map[self::ON_DELETE]) {
                     case self::CASCADE:
-                    	$where = array();
+                        $where = array();
                         for ($i = 0; $i < count($map[self::COLUMNS]); ++$i) {
                             $col = $this->_db->foldCase($map[self::COLUMNS][$i]);
                             $refCol = $this->_db->foldCase($map[self::REF_COLUMNS][$i]);
@@ -954,6 +1216,7 @@ abstract class Zend_Db_Table_Abstract
      */
     public function find()
     {
+        $this->_setupPrimaryKey();
         $args = func_get_args();
         $keyNames = array_values((array) $this->_primary);
 
@@ -970,6 +1233,7 @@ abstract class Zend_Db_Table_Abstract
         $whereList = array();
         $numberTerms = 0;
         foreach ($args as $keyPosition => $keyValues) {
+            $keyValuesCount = count($keyValues);
             // Coerce the values to an array.
             // Don't simply typecast to array, because the values
             // might be Zend_Db_Expr objects.
@@ -977,12 +1241,13 @@ abstract class Zend_Db_Table_Abstract
                 $keyValues = array($keyValues);
             }
             if ($numberTerms == 0) {
-                $numberTerms = count($keyValues);
-            } else if (count($keyValues) != $numberTerms) {
+                $numberTerms = $keyValuesCount;
+            } else if ($keyValuesCount != $numberTerms) {
                 require_once 'Zend/Db/Table/Exception.php';
                 throw new Zend_Db_Table_Exception("Missing value(s) for the primary key");
             }
-            for ($i = 0; $i < count($keyValues); ++$i) {
+            $keyValues = array_values($keyValues);
+            for ($i = 0; $i < $keyValuesCount; ++$i) {
                 if (!isset($whereList[$i])) {
                     $whereList[$i] = array();
                 }
@@ -993,11 +1258,11 @@ abstract class Zend_Db_Table_Abstract
         $whereClause = null;
         if (count($whereList)) {
             $whereOrTerms = array();
+            $tableName = $this->_db->quoteTableAs($this->_name, null, true);
             foreach ($whereList as $keyValueSets) {
                 $whereAndTerms = array();
                 foreach ($keyValueSets as $keyPosition => $keyValue) {
                     $type = $this->_metadata[$keyNames[$keyPosition]]['DATA_TYPE'];
-                    $tableName = $this->_db->quoteTableAs($this->_name, null, true);
                     $columnName = $this->_db->quoteIdentifier($keyNames[$keyPosition], true);
                     $whereAndTerms[] = $this->_db->quoteInto(
                         $tableName . '.' . $columnName . ' = ?',
@@ -1008,6 +1273,16 @@ abstract class Zend_Db_Table_Abstract
             $whereClause = '(' . implode(' OR ', $whereOrTerms) . ')';
         }
 
+        // issue ZF-5775 (empty where clause should return empty rowset)
+        if ($whereClause == null) {
+            $rowsetClass = $this->getRowsetClass();
+            if (!class_exists($rowsetClass)) {
+                require_once 'Zend/Loader.php';
+                Zend_Loader::loadClass($rowsetClass);
+            }
+            return new $rowsetClass(array('table' => $this, 'rowClass' => $this->getRowClass(), 'stored' => true));
+        }
+
         return $this->fetchAll($whereClause);
     }
 
@@ -1049,21 +1324,25 @@ abstract class Zend_Db_Table_Abstract
             'table'    => $this,
             'data'     => $rows,
             'readOnly' => $select->isReadOnly(),
-            'rowClass' => $this->_rowClass,
+            'rowClass' => $this->getRowClass(),
             'stored'   => true
         );
 
-        @Zend_Loader::loadClass($this->_rowsetClass);
-        return new $this->_rowsetClass($data);
+        $rowsetClass = $this->getRowsetClass();
+        if (!class_exists($rowsetClass)) {
+            require_once 'Zend/Loader.php';
+            Zend_Loader::loadClass($rowsetClass);
+        }
+        return new $rowsetClass($data);
     }
 
     /**
      * Fetches one row in an object of type Zend_Db_Table_Row_Abstract,
-     * or returns Boolean false if no row matches the specified criteria.
+     * or returns null if no row matches the specified criteria.
      *
      * @param string|array|Zend_Db_Table_Select $where  OPTIONAL An SQL WHERE clause or Zend_Db_Table_Select object.
      * @param string|array                      $order  OPTIONAL An SQL ORDER clause.
-     * @return Zend_Db_Table_Row_Abstract The row results per the
+     * @return Zend_Db_Table_Row_Abstract|null The row results per the
      *     Zend_Db_Adapter fetch mode, or null if no row found.
      */
     public function fetchRow($where = null, $order = null)
@@ -1098,8 +1377,12 @@ abstract class Zend_Db_Table_Abstract
             'stored'  => true
         );
 
-        @Zend_Loader::loadClass($this->_rowClass);
-        return new $this->_rowClass($data);
+        $rowClass = $this->getRowClass();
+        if (!class_exists($rowClass)) {
+            require_once 'Zend/Loader.php';
+            Zend_Loader::loadClass($rowClass);
+        }
+        return new $rowClass($data);
     }
 
     /**
@@ -1117,12 +1400,39 @@ abstract class Zend_Db_Table_Abstract
      * Fetches a new blank row (not from the database).
      *
      * @param  array $data OPTIONAL data to populate in the new row.
+     * @param  string $defaultSource OPTIONAL flag to force default values into new row
      * @return Zend_Db_Table_Row_Abstract
      */
-    public function createRow(array $data = array())
+    public function createRow(array $data = array(), $defaultSource = null)
     {
-        $defaults = array_combine($this->_cols, array_fill(0, count($this->_cols), null));
-        $data = array_intersect_key($data, $defaults);
+        $cols     = $this->_getCols();
+        $defaults = array_combine($cols, array_fill(0, count($cols), null));
+
+        // nothing provided at call-time, take the class value
+        if ($defaultSource == null) {
+            $defaultSource = $this->_defaultSource;
+        }
+
+        if (!in_array($defaultSource, array(self::DEFAULT_CLASS, self::DEFAULT_DB, self::DEFAULT_NONE))) {
+            $defaultSource = self::DEFAULT_NONE;
+        }
+
+        if ($defaultSource == self::DEFAULT_DB) {
+            foreach ($this->_metadata as $metadataName => $metadata) {
+                if (($metadata['DEFAULT'] != null) &&
+                    ($metadata['NULLABLE'] !== true || ($metadata['NULLABLE'] === true && isset($this->_defaultValues[$metadataName]) && $this->_defaultValues[$metadataName] === true)) &&
+                    (!(isset($this->_defaultValues[$metadataName]) && $this->_defaultValues[$metadataName] === false))) {
+                    $defaults[$metadataName] = $metadata['DEFAULT'];
+                }
+            }
+        } elseif ($defaultSource == self::DEFAULT_CLASS && $this->_defaultValues) {
+            foreach ($this->_defaultValues as $defaultName => $defaultValue) {
+                if (array_key_exists($defaultName, $defaults)) {
+                    $defaults[$defaultName] = $defaultValue;
+                }
+            }
+        }
+
         $config = array(
             'table'    => $this,
             'data'     => $defaults,
@@ -1130,8 +1440,12 @@ abstract class Zend_Db_Table_Abstract
             'stored'   => false
         );
 
-        @Zend_Loader::loadClass($this->_rowClass);
-        $row = new $this->_rowClass($config);
+        $rowClass = $this->getRowClass();
+        if (!class_exists($rowClass)) {
+            require_once 'Zend/Loader.php';
+            Zend_Loader::loadClass($rowClass);
+        }
+        $row = new $rowClass($config);
         $row->setFromArray($data);
         return $row;
     }
