@@ -15,8 +15,10 @@
  *
  * @category   Zend
  * @package    Zend_Gdata
- * @copyright  Copyright (c) 2005-2008 Zend Technologies USA Inc. (http://www.zend.com)
+ * @subpackage Photos
+ * @copyright  Copyright (c) 2005-2010 Zend Technologies USA Inc. (http://www.zend.com)
  * @license    http://framework.zend.com/license/new-bsd     New BSD License
+ * @version    $Id: PhotoEntry.php 20096 2010-01-06 02:05:09Z bkarwin $
  */
 
 /**
@@ -101,8 +103,8 @@ require_once 'Zend/Gdata/App/Extension/Category.php';
 
 /**
  * Data model class for a Comment Entry.
- * 
- * To transfer user entries to and from the servers, including 
+ *
+ * To transfer user entries to and from the servers, including
  * creating new entries, refer to the service class,
  * Zend_Gdata_Photos.
  *
@@ -110,118 +112,117 @@ require_once 'Zend/Gdata/App/Extension/Category.php';
  *
  * @category   Zend
  * @package    Zend_Gdata
- * @copyright  Copyright (c) 2005-2008 Zend Technologies USA Inc. (http://www.zend.com)
+ * @subpackage Photos
+ * @copyright  Copyright (c) 2005-2010 Zend Technologies USA Inc. (http://www.zend.com)
  * @license    http://framework.zend.com/license/new-bsd     New BSD License
  */
 class Zend_Gdata_Photos_PhotoEntry extends Zend_Gdata_Media_Entry
 {
 
     protected $_entryClassName = 'Zend_Gdata_Photos_PhotoEntry';
-    
+
     /**
      * gphoto:id element
      *
      * @var Zend_Gdata_Photos_Extension_Id
      */
     protected $_gphotoId = null;
-    
+
     /**
      * gphoto:albumid element
      *
      * @var Zend_Gdata_Photos_Extension_AlbumId
      */
     protected $_gphotoAlbumId = null;
-    
+
     /**
      * gphoto:version element
      *
      * @var Zend_Gdata_Photos_Extension_Version
      */
     protected $_gphotoVersion = null;
-    
+
     /**
      * gphoto:width element
      *
      * @var Zend_Gdata_Photos_Extension_Width
      */
     protected $_gphotoWidth = null;
-    
+
     /**
      * gphoto:height element
      *
      * @var Zend_Gdata_Photos_Extension_Height
      */
     protected $_gphotoHeight = null;
-    
+
     /**
      * gphoto:size element
      *
      * @var Zend_Gdata_Photos_Extension_Size
      */
     protected $_gphotoSize = null;
-    
+
     /**
      * gphoto:client element
      *
      * @var Zend_Gdata_Photos_Extension_Client
      */
     protected $_gphotoClient = null;
-    
+
     /**
      * gphoto:checksum element
      *
      * @var Zend_Gdata_Photos_Extension_Checksum
      */
     protected $_gphotoChecksum = null;
-    
+
     /**
      * gphoto:timestamp element
      *
      * @var Zend_Gdata_Photos_Extension_Timestamp
      */
     protected $_gphotoTimestamp = null;
-    
+
     /**
      * gphoto:commentCount element
      *
      * @var Zend_Gdata_Photos_Extension_CommentCount
      */
     protected $_gphotoCommentCount = null;
-    
+
     /**
      * gphoto:commentingEnabled element
      *
      * @var Zend_Gdata_Photos_Extension_CommentingEnabled
      */
     protected $_gphotoCommentingEnabled = null;
-    
+
     /**
      * exif:tags element
      *
      * @var Zend_Gdata_Exif_Extension_Tags
      */
     protected $_exifTags = null;
-    
+
     /**
      * georss:where element
      *
      * @var Zend_Gdata_Geo_Extension_GeoRssWhere
      */
     protected $_geoRssWhere = null;
-    
+
     /**
      * Create a new instance.
-     * 
+     *
      * @param DOMElement $element (optional) DOMElement from which this
      *          object should be constructed.
      */
     public function __construct($element = null)
     {
-        foreach (Zend_Gdata_Photos::$namespaces as $nsPrefix => $nsUri) {
-            $this->registerNamespace($nsPrefix, $nsUri);
-        }
+        $this->registerAllNamespaces(Zend_Gdata_Photos::$namespaces);
         parent::__construct($element);
-        
+
         $category = new Zend_Gdata_App_Extension_Category(
             'http://schemas.google.com/photos/2007#photo',
             'http://schemas.google.com/g/2005#kind');
@@ -237,9 +238,9 @@ class Zend_Gdata_Photos_PhotoEntry extends Zend_Gdata_Media_Entry
      * @return DOMElement The DOMElement representing this element and all
      *          child properties.
      */
-    public function getDOM($doc = null)
+    public function getDOM($doc = null, $majorVersion = 1, $minorVersion = null)
     {
-        $element = parent::getDOM($doc);
+        $element = parent::getDOM($doc, $majorVersion, $minorVersion);
         if ($this->_gphotoAlbumId !== null) {
             $element->appendChild($this->_gphotoAlbumId->getDOM($element->ownerDocument));
         }
@@ -291,69 +292,69 @@ class Zend_Gdata_Photos_PhotoEntry extends Zend_Gdata_Media_Entry
     protected function takeChildFromDOM($child)
     {
         $absoluteNodeName = $child->namespaceURI . ':' . $child->localName;
-        
+
         switch ($absoluteNodeName) {
-            case $this->lookupNamespace('gphoto') . ':' . 'albumid'; 
+            case $this->lookupNamespace('gphoto') . ':' . 'albumid';
                 $albumId = new Zend_Gdata_Photos_Extension_AlbumId();
                 $albumId->transferFromDOM($child);
                 $this->_gphotoAlbumId = $albumId;
                 break;
-            case $this->lookupNamespace('gphoto') . ':' . 'id'; 
+            case $this->lookupNamespace('gphoto') . ':' . 'id';
                 $id = new Zend_Gdata_Photos_Extension_Id();
                 $id->transferFromDOM($child);
                 $this->_gphotoId = $id;
                 break;
-            case $this->lookupNamespace('gphoto') . ':' . 'version'; 
+            case $this->lookupNamespace('gphoto') . ':' . 'version';
                 $version = new Zend_Gdata_Photos_Extension_Version();
                 $version->transferFromDOM($child);
                 $this->_gphotoVersion = $version;
                 break;
-            case $this->lookupNamespace('gphoto') . ':' . 'width'; 
+            case $this->lookupNamespace('gphoto') . ':' . 'width';
                 $width = new Zend_Gdata_Photos_Extension_Width();
                 $width->transferFromDOM($child);
                 $this->_gphotoWidth = $width;
                 break;
-            case $this->lookupNamespace('gphoto') . ':' . 'height'; 
+            case $this->lookupNamespace('gphoto') . ':' . 'height';
                 $height = new Zend_Gdata_Photos_Extension_Height();
                 $height->transferFromDOM($child);
                 $this->_gphotoHeight = $height;
                 break;
-            case $this->lookupNamespace('gphoto') . ':' . 'size'; 
+            case $this->lookupNamespace('gphoto') . ':' . 'size';
                 $size = new Zend_Gdata_Photos_Extension_Size();
                 $size->transferFromDOM($child);
                 $this->_gphotoSize = $size;
                 break;
-            case $this->lookupNamespace('gphoto') . ':' . 'client'; 
+            case $this->lookupNamespace('gphoto') . ':' . 'client';
                 $client = new Zend_Gdata_Photos_Extension_Client();
                 $client->transferFromDOM($child);
                 $this->_gphotoClient = $client;
                 break;
-            case $this->lookupNamespace('gphoto') . ':' . 'checksum'; 
+            case $this->lookupNamespace('gphoto') . ':' . 'checksum';
                 $checksum = new Zend_Gdata_Photos_Extension_Checksum();
                 $checksum->transferFromDOM($child);
                 $this->_gphotoChecksum = $checksum;
                 break;
-            case $this->lookupNamespace('gphoto') . ':' . 'timestamp'; 
+            case $this->lookupNamespace('gphoto') . ':' . 'timestamp';
                 $timestamp = new Zend_Gdata_Photos_Extension_Timestamp();
                 $timestamp->transferFromDOM($child);
                 $this->_gphotoTimestamp = $timestamp;
                 break;
-            case $this->lookupNamespace('gphoto') . ':' . 'commentingEnabled'; 
+            case $this->lookupNamespace('gphoto') . ':' . 'commentingEnabled';
                 $commentingEnabled = new Zend_Gdata_Photos_Extension_CommentingEnabled();
                 $commentingEnabled->transferFromDOM($child);
                 $this->_gphotoCommentingEnabled = $commentingEnabled;
                 break;
-            case $this->lookupNamespace('gphoto') . ':' . 'commentCount'; 
+            case $this->lookupNamespace('gphoto') . ':' . 'commentCount';
                 $commentCount = new Zend_Gdata_Photos_Extension_CommentCount();
                 $commentCount->transferFromDOM($child);
                 $this->_gphotoCommentCount = $commentCount;
                 break;
-            case $this->lookupNamespace('exif') . ':' . 'tags'; 
+            case $this->lookupNamespace('exif') . ':' . 'tags';
                 $exifTags = new Zend_Gdata_Exif_Extension_Tags();
                 $exifTags->transferFromDOM($child);
                 $this->_exifTags = $exifTags;
                 break;
-            case $this->lookupNamespace('georss') . ':' . 'where'; 
+            case $this->lookupNamespace('georss') . ':' . 'where';
                 $geoRssWhere = new Zend_Gdata_Geo_Extension_GeoRssWhere();
                 $geoRssWhere->transferFromDOM($child);
                 $this->_geoRssWhere = $geoRssWhere;
@@ -361,10 +362,10 @@ class Zend_Gdata_Photos_PhotoEntry extends Zend_Gdata_Media_Entry
             default:
                 parent::takeChildFromDOM($child);
                 break;
-            
+
         }
     }
-    
+
     /**
      * Get the value for this element's gphoto:albumid attribute.
      *
@@ -387,7 +388,7 @@ class Zend_Gdata_Photos_PhotoEntry extends Zend_Gdata_Media_Entry
         $this->_gphotoAlbumId = $value;
         return $this;
     }
-    
+
     /**
      * Get the value for this element's gphoto:id attribute.
      *
@@ -594,7 +595,7 @@ class Zend_Gdata_Photos_PhotoEntry extends Zend_Gdata_Media_Entry
         $this->_gphotoCommentCount = $value;
         return $this;
     }
-    
+
     /**
      * Get the value for this element's gphoto:commentingEnabled attribute.
      *
@@ -617,7 +618,7 @@ class Zend_Gdata_Photos_PhotoEntry extends Zend_Gdata_Media_Entry
         $this->_gphotoCommentingEnabled = $value;
         return $this;
     }
-    
+
     /**
      * Get the value for this element's exif:tags attribute.
      *
@@ -640,7 +641,7 @@ class Zend_Gdata_Photos_PhotoEntry extends Zend_Gdata_Media_Entry
         $this->_exifTags = $value;
         return $this;
     }
-    
+
     /**
      * Get the value for this element's georss:where attribute.
      *
