@@ -11,13 +11,13 @@
  * If you did not receive a copy of the license and are unable to
  * obtain it through the world-wide-web, please send an email
  * to license@zend.com so we can send you a copy immediately.
- * 
+ *
  * @category   Zend
  * @package    Zend_Mail
  * @subpackage Storage
- * @copyright  Copyright (c) 2005-2008 Zend Technologies USA Inc. (http://www.zend.com)
+ * @copyright  Copyright (c) 2005-2010 Zend Technologies USA Inc. (http://www.zend.com)
  * @license    http://framework.zend.com/license/new-bsd     New BSD License
- * @version    $Id: Imap.php 9099 2008-03-30 19:35:47Z thomas $
+ * @version    $Id: Imap.php 20096 2010-01-06 02:05:09Z bkarwin $
  */
 
 
@@ -60,7 +60,7 @@ require_once 'Zend/Mail/Storage.php';
  * @category   Zend
  * @package    Zend_Mail
  * @subpackage Storage
- * @copyright  Copyright (c) 2005-2008 Zend Technologies USA Inc. (http://www.zend.com)
+ * @copyright  Copyright (c) 2005-2010 Zend Technologies USA Inc. (http://www.zend.com)
  * @license    http://framework.zend.com/license/new-bsd     New BSD License
  */
 class Zend_Mail_Storage_Imap extends Zend_Mail_Storage_Abstract
@@ -120,19 +120,19 @@ class Zend_Mail_Storage_Imap extends Zend_Mail_Storage_Abstract
             throw new Zend_Mail_Storage_Exception('No selected folder to count');
         }
 
-    	if ($flags === null) {
-    		return count($this->_protocol->search(array('ALL')));
-    	}
-    	
-    	$params = array();
-    	foreach ((array)$flags as $flag) {
-    		if (isset(self::$_searchFlags[$flag])) {
-    			$params[] = self::$_searchFlags[$flag];
-    		} else {
-    			$params[] = 'KEYWORD';
-    			$params[] = $this->_protocol->escapeString($flag);
-    		}
-    	}
+        if ($flags === null) {
+            return count($this->_protocol->search(array('ALL')));
+        }
+
+        $params = array();
+        foreach ((array)$flags as $flag) {
+            if (isset(self::$_searchFlags[$flag])) {
+                $params[] = self::$_searchFlags[$flag];
+            } else {
+                $params[] = 'KEYWORD';
+                $params[] = $this->_protocol->escapeString($flag);
+            }
+        }
         return count($this->_protocol->search($params));
     }
 
@@ -251,7 +251,7 @@ class Zend_Mail_Storage_Imap extends Zend_Mail_Storage_Abstract
                  * @see Zend_Mail_Storage_Exception
                  */
                 require_once 'Zend/Mail/Storage/Exception.php';
-                throw new Zend_Mail_Storage_Exception('cannot select INBOX, is this a valid transport?');
+                throw new Zend_Mail_Storage_Exception('cannot select INBOX, is this a valid transport?', 0, $e);
             }
             return;
         }
@@ -617,8 +617,8 @@ class Zend_Mail_Storage_Imap extends Zend_Mail_Storage_Abstract
      * @throws Zend_Mail_Storage_Exception
      */
     public function moveMessage($id, $folder) {
-    	$this->copyMessage($id, $folder);
-    	$this->removeMessage($id);
+        $this->copyMessage($id, $folder);
+        $this->removeMessage($id);
     }
 
     /**
